/* THIS IS NOT AN X CONSORTIUM STANDARD */

#define NEED_REPLIES
#include <X11/Xlibint.h>
#include "X11/extensions/Xclipliststr.h"
#include <X11/extensions/Xext.h>
#include <X11/extensions/extutil.h>

#include <stdio.h>

static XExtensionInfo _xf86dh_info_data;
static XExtensionInfo *xf86dh_info = &_xf86dh_info_data;
static char *xf86dh_extension_name = XCLIPLISTNAME;

#define XClipListCheckExtension(dpy,i,val) \
  XextCheckExtension (dpy, i, xf86dh_extension_name, val)

/*****************************************************************************
 *                                                                           *
 *			   private utility routines                          *
 *                                                                           *
 *****************************************************************************/

static int close_display(Display *dpy, XExtCodes *extCodes);
static /* const */ XExtensionHooks xf86dh_extension_hooks = {
    NULL,				/* create_gc */
    NULL,				/* copy_gc */
    NULL,				/* flush_gc */
    NULL,				/* free_gc */
    NULL,				/* create_font */
    NULL,				/* free_font */
    close_display,			/* close_display */
    NULL,				/* wire_to_event */
    NULL,				/* event_to_wire */
    NULL,				/* error */
    NULL,				/* error_string */
};

static XEXT_GENERATE_FIND_DISPLAY (find_display, xf86dh_info, 
				   xf86dh_extension_name, 
				   &xf86dh_extension_hooks, 
				   0, NULL)

static XEXT_GENERATE_CLOSE_DISPLAY (close_display, xf86dh_info)


/*****************************************************************************
 *                                                                           *
 *		    public XClipList Extension routines                      *
 *                                                                           *
 *****************************************************************************/

#if 0
#include <stdio.h>
#define TRACE(msg)  fprintf(stderr,"XClipList%s\n", msg);
#else
#define TRACE(msg)
#endif


Bool
XClipListQueryExtension(Display *dpy,
                        int *event_basep,
                        int *error_basep)
{
    XExtDisplayInfo *info = find_display (dpy);

    TRACE("QueryExtension...");
    if (XextHasExtension(info)) {
	*event_basep = info->codes->first_event;
	*error_basep = info->codes->first_error;
        TRACE("QueryExtension... return True");
	return True;
    }
    else {
        TRACE("QueryExtension... return False");
	return False;
    }
}


Bool
XClipListQueryVersion(Display* dpy,
                      int *majorVersion, 
                      int *minorVersion,
                      int *patchVersion)
{
    XExtDisplayInfo *info = find_display (dpy);
    xXClipListQueryVersionReply rep;
    xXClipListQueryVersionReq *req;

    TRACE("QueryVersion...");
    XClipListCheckExtension (dpy, info, False);

    LockDisplay(dpy);
    GetReq(XClipListQueryVersion, req);
    req->reqType = info->codes->major_opcode;
    req->dhReqType = X_XClipListQueryVersion;
    if (!_XReply(dpy, (xReply *)&rep, 0, xFalse)) {
	UnlockDisplay(dpy);
	SyncHandle();
        TRACE("QueryVersion... return False");
	return False;
    }
    *majorVersion = rep.majorVersion;
    *minorVersion = rep.minorVersion;
    *patchVersion = rep.patchVersion;
    UnlockDisplay(dpy);
    SyncHandle();
    TRACE("QueryVersion... return True");
    return True;
}


BoxPtr
XGetClipList(Display *dpy,
             XID wid,
             int *num)
{
    XExtDisplayInfo *info = find_display (dpy);
    xXGetClipListReply rep;
    xXGetClipListReq *req;
    BoxPtr ret = NULL;

    *num = 0;

    TRACE("Mode...");
    XClipListCheckExtension (dpy, info, False);

    LockDisplay(dpy);
    GetReq(XGetClipList, req);

    req->reqType = info->codes->major_opcode;
    req->dhReqType = X_XGetClipList;
    req->windowid = wid;

    if (!_XReply(dpy, (xReply *)&rep, 0, xFalse)) 
    {
	UnlockDisplay(dpy);
	SyncHandle();
        TRACE("Mode... return False");
	return False;
    }

    if (rep.num_entries) {
        int size = (rep.num_entries * sizeof(BoxRec));

        if ((ret = Xmalloc(size))) {
	    BoxRec region;
	    int i;
	
            for (i = 0; i < rep.num_entries; i++) {
                _XRead(dpy, (char*)(&region), sizeof(BoxRec));
                memcpy(&(ret[i]), &(region), sizeof(BoxRec));
                (*num)++;
            }
        }
        else {
           _XEatData(dpy, rep.length << 2);
        }
    }

    UnlockDisplay(dpy);
    SyncHandle();

    return ret;
}
